

#############################################################
#####         low level utility routines       ##############
#############################################################
  
namevalue <- function(nam, val) {
  paste(nam, "=\"", val, "\"", sep="") 
  }

numbered.id <- function(before, g, after) {
  paste("id=\"", before, g, after, "\"", sep="") 
  }
  
id <- function(id) { 
  paste("id=\"", id, "\"", sep="")
  }
  
  
idref <- function(element, id) {
  paste("<", element, " idref=\"", id, "\" />", sep="") 
  }  

numbered.idref <- function(element, before, g, after) {
  paste("<", element, " idref=\"", before, g, after, "\" />", sep="") 
  }  
  

catln <- function(indent, ...) {
  if (indent == 0) {
    cat(..., "\n", sep="")
    } else {
    tabs <- paste(rep("\t", indent), sep="", collapse="")
    cat(tabs, ..., "\n") 
    }
  }
 
commentln <- function(txt) {
cat("<!-- ", txt, " -->\n"); 
}

  
#############################################################
##########                  The data           ##############
#############################################################
 



  
  
#############################################################
##########           models and priors         ##############
#############################################################
  

make.patterns <- function(g) {
  catln(1, "<patterns", numbered.id("", g, ".patterns"), namevalue("from", 1), ">")
  catln(2, numbered.idref("alignment", "alignment", g, ""))
  catln(1, "</patterns>")
  }
  
make.constantSize <- function() { 
 	catln(1, "<constantSize",  id("constant"), namevalue("units", "substitutions"), ">")
  catln(2, "<populationSize>")
  catln(3, "<parameter",  id("constant.popSize"),  namevalue("value", 0.016),
                 namevalue("lower", 0.0), namevalue("upper", "Infinity"), "/>")
  catln(2, "</populationSize>")
  catln(1, "</constantSize>")
 }
  
make.coalescentTree <- function(g, roothgt) {
  catln(1, "<coalescentTree ", numbered.id("", g, ".startingTree"),
          namevalue("rootHeight", roothgt), ">")
  catln(2, idref("taxa", "taxa"))
  catln(2, idref("constantSize", "constant"))
  catln(1, "</coalescentTree>")
  }


make.treeModel <- function(g) {
  catln(1, "<treeModel", numbered.id("", g, ".treeModel"), ">")
  catln(2, numbered.idref("coalescentTree", "", g, ".startingTree"))
  
  catln(2, "<rootHeight>")
  catln(3, "<parameter", numbered.id("", g, ".treeModel.rootHeight"), "/>")
  catln(2, "</rootHeight>")
  
  catln(2, "<nodeHeights internalNodes=\"true\" >")
  catln(3, "<parameter", numbered.id("", g, ".treeModel.internalNodeHeights"), "/>")
  catln(2, "</nodeHeights>")

  catln(2, "<nodeHeights internalNodes=\"true\" rootNode=\"true\" >")
  catln(3, "<parameter", numbered.id("", g, ".treeModel.allInternalNodeHeights"), "/>")
  catln(2, "</nodeHeights>")
  
  catln(1, "</treeModel>")
  }



make.strictClockBranchRate <- function(g) {
  catln(1, "<strictClockBranchRates", numbered.id("", g, ".branchRates"), ">")
  catln(2, "<rate>")
  catln(3, "<parameter", numbered.id("", g, ".clock.rate"), 
           namevalue("value", 1.0), namevalue("lower", 0.0),
           namevalue("upper", "Infinity"), "/>") 
  catln(2, "</rate>")
  catln(1, "</strictClockBranchRates>")
  }


  
make.HKYModel <- function(g) {
  catln(1, "<HKYModel", numbered.id("", g, ".hky"), ">")
  catln(2, "<frequencies>")
  catln(3, "<frequencyModel", namevalue("dataType", "nucleotide"), ">")
  catln(4, "<frequencies>")
  catln(5, "<parameter", numbered.id("", g, ".frequencies"), 
                 namevalue("value", "0.25 0.25 0.25 0.25"), "/>")
  catln(4, "</frequencies>")
  catln(3, "</frequencyModel>")
  catln(2, "</frequencies>")
  catln(2, "<kappa>")
  catln(3, "<parameter", numbered.id("", g, ".kappa"), namevalue("value", 2.0),
              namevalue("lower", 0), namevalue("upper", "Infinity"), "/>")
  catln(2, "</kappa>")
  catln(1, "</HKYModel>")
  }


make.siteModel <- function(g) {
  catln(1, "<siteModel", numbered.id("", g, ".siteModel"), ">")
  catln(2, "<substitutionModel>")
  catln(3, numbered.idref("HKYModel", "", g, ".hky"))
  catln(2, "</substitutionModel>")
  catln(1, "</siteModel>")
  }


make.treeLikelihood <- function(g) {
  catln(1, "<treeLikelihood", numbered.id("", g, ".treeLikelihood"), 
                namevalue("useAmbiguities", "false"), ">")
  catln(2, numbered.idref("patterns", "", g, ".patterns"))
  catln(2, numbered.idref("treeModel", "", g, ".treeModel"))
  catln(2, numbered.idref("siteModel", "", g, ".siteModel"))
  catln(2, numbered.idref("strictClockBranchRates", "", g, ".branchRates"))
  catln(1, "</treeLikelihood>")
  }


  
make.alloppSpecies <- function(nofgenes, taxastruct, mingenenodeheight) {
  catln(1, "<alloppSpecies", id("alloppSpecies"), 
               namevalue("minGeneNodeHeight", mingenenodeheight), ">")
  for (sp in 1:length(taxastruct)) {
    spname <- taxastruct[[sp]]$sp
    ploidylevel <- 2 * length(taxastruct[[sp]]$sqs)
    catln(2, "<apsp", id(spname), namevalue("ploidylevel", ploidylevel), ">")
    for (iv in 1:length(taxastruct[[sp]]$indivs)) {
      ivname <- paste(taxastruct[[sp]]$indivs[iv], spname, sep="")
      catln(3, "<individual", id(ivname), ">")
      for (sq in 1:length(taxastruct[[sp]]$sqs)) {
        txname <- paste(ivname, taxastruct[[sp]]$sqs[sq], sep="")
        catln(4, idref("taxon", txname)) 
        }
      catln(3, "</individual>")
      }
    catln(2, "</apsp>")
    }           
  catln(2, "<geneTrees", id("geneTrees"), ">")
  for (g in 1:nofgenes) {
    catln(3, numbered.idref("treeModel", "", g, ".treeModel"))
    }
  catln(2, "</geneTrees>")
  catln(1, "</alloppSpecies>")
  } 
  
 
 # grjtodo allow for > 1 hybridizations 
make.alloppSpeciesNetwork <- function(initpopval) { 
  catln(1, "<alloppSpeciesNetwork", id("apspnetwork"), "oneHybridization=\"true\" >")
  catln(2, idref("alloppSpecies", "alloppSpecies")) 
  catln(2, "<sppSplitPopulations", namevalue("value", initpopval), ">")
  catln(3, "<parameter", id("apspnetwork.splitPopSize"), "/>")
  catln(2, "</sppSplitPopulations>") 
  catln(1, "</alloppSpeciesNetwork>")
  }


make.alloppNetworkPriorModel <- function() { 
  catln(1, "<alloppNetworkPriorModel", id("one.rate"), 
                   namevalue("units", "substitutions"), ">")
  catln(2, "<eventRate>")
  catln(3, "<parameter", id("apspnetwork.prior.eventRate"), 
             namevalue("value", 100), namevalue("lower", 0.0), namevalue("upper", "Infinity"),"/>")
  catln(2, "</eventRate>")
  catln(1, "</alloppNetworkPriorModel>")
  }


make.apspNetworkPrior <- function() { 
  catln(1, "<apspNetworkPrior", id("apspnetwork.prior"), ">")
  catln(2, "<model>")
  catln(3, idref("alloppNetworkPriorModel", "one.rate"))
  catln(2, "</model>")
  catln(2, "<apspNetwork>")
  catln(3, idref("alloppSpeciesNetwork", "apspnetwork"))
  catln(2, "</apspNetwork>")
  catln(1, "</apspNetworkPrior>")
  }
  
make.apspCoalescent <- function() { 
  catln(1, "<apspCoalescent", id("apsp.coalescent"), ">")
  catln(2, idref("alloppSpecies", "alloppSpecies"))
  catln(2, idref("alloppSpeciesNetwork", "apspnetwork"))
  catln(1, "</apspCoalescent>")    
  }






make.gammaDistributionModel <- function(shape, initpopval, first) {
  catln(3, "<gammaDistributionModel>")
  catln(4, "<shape>")
  catln(5, shape)
  catln(4, "</shape>")
  catln(4, "<scale>")
  if (first) {
    catln(5, "<parameter", id("species.popMean"), namevalue("value", initpopval), 
               namevalue("lower", 0.0), namevalue("upper", "Infinity"),"/>")
    } else {
    catln(5, idref("parameter", "species.popMean"))
    }
  catln(4, "</scale>")
  catln(3, "</gammaDistributionModel>")
  }



 # this for is two diploids case
 make.mixedDistributionLikelihood <- function(nofdiploids, noftetraploids, 
                                         noftetratrees, initpopval) { 
   flags <- ""
   for (i in 1:(nofdiploids + noftetraploids)) {
     flags <- paste(flags, "1")
     }
   for (i in 1:(2*nofdiploids + 2*noftetraploids + noftetratrees - 2)) {
     flags <- paste(flags, "0")
     }
   catln(1, "<mixedDistributionLikelihood", id("species.popSize"), ">")
   catln(2, "<distribution0>")
   make.gammaDistributionModel(2, initpopval, TRUE)
   catln(2, "</distribution0>")
   catln(2, "<distribution1>")
   make.gammaDistributionModel(4, initpopval, FALSE)
   catln(2, "</distribution1>")
   catln(2, "<data>")
   catln(3, idref("parameter", "apspnetwork.splitPopSize"))
   catln(2, "</data>")
   catln(2, "<indicators>")
   catln(3, "<parameter",  namevalue("value", flags), "/>")
   catln(2, "</indicators>")  
   catln(1, "</mixedDistributionLikelihood>")
   }
   
   



#############################################################
##########            MCMC operators           ##############
#############################################################
 


make.general.numbered.scaleOperator <- function(g, sfactor, weight, before, after) {
  catln(2, "<scaleOperator", namevalue("scaleFactor", sfactor),
         namevalue("weight", weight), ">")
  catln(3, numbered.idref("parameter", before, g, after))
  catln(2, "</scaleOperator>")
  }		
		
make.general.scaleOperator <- function(sfactor, weight, param) {
  catln(2, "<scaleOperator", namevalue("scaleFactor", sfactor),
         namevalue("weight", weight), ">")
  catln(3, idref("parameter", param))
  catln(2, "</scaleOperator>")
  }			
		
		
make.scaleOperator.kappa <- function(g) {
  make.general.numbered.scaleOperator(g, 0.75, 1, "", ".kappa")
  }
  

make.scaleOperator.clockrate <- function(g) {
  make.general.numbered.scaleOperator(g, 0.75, 1, "", ".clock.rate")
  }  
	
make.deltaExchange.frequencies <- function(g) {
  catln(2, "<deltaExchange", namevalue("delta", 0.01),
         namevalue("weight", 1), ">")
  catln(3, numbered.idref("parameter", "", g, ".frequencies"))
  catln(2, "</deltaExchange>")  
  }
  

make.subtreeSlide.treeModel <- function(g) {
  catln(2, "<subtreeSlide", namevalue("size", .002), 
       namevalue("gaussian", "true"), namevalue("weight", 15), ">")
  catln(3, numbered.idref("treeModel", "", g, ".treeModel"))
  catln(2, "</subtreeSlide>") 
  }
  

  
make.narrowExchange.treeModel <- function(g) {
  catln(2, "<narrowExchange", namevalue("weight", 15), ">")
  catln(3, numbered.idref("treeModel", "", g, ".treeModel"))
  catln(2, "</narrowExchange>") 
  }
  
   
make.wideExchange.treeModel <- function(g) {
  catln(2, "<wideExchange", namevalue("weight", 3), ">")
  catln(3, numbered.idref("treeModel", "", g, ".treeModel"))
  catln(2, "</wideExchange>") 
  }
  
   
make.wilsonBalding.treeModel <- function(g) {
  catln(2, "<wilsonBalding", namevalue("weight", 3), ">")
  catln(3, numbered.idref("treeModel", "", g, ".treeModel"))
  catln(2, "</wilsonBalding>") 
  
  }
  
  
make.scaleOperator.treeModel.rootHeight <- function(g) {
  make.general.numbered.scaleOperator(g, 0.75, 3, "", ".treeModel.rootHeight")
  }
  

 
make.uniformOperator.treeModel.internalNodeHeights <- function(g) {
  catln(2, "<uniformOperator", namevalue("weight", 30), ">")
  catln(3, numbered.idref("parameter", "", g, ".treeModel.internalNodeHeights"))
  catln(2, "</uniformOperator>")
  }
  
   

   

   
   
make.upDownOperator.clockrate.allInternalNodeHeights <- function(g) {
  catln(2, "<upDownOperator", namevalue("scaleFactor", 0.75), 
           namevalue("weight", 30), ">")
  catln(3, "<up>")
  if (g>1) {
    catln(4, numbered.idref("parameter", "", g, ".clock.rate")) 
    }  
  catln(3, "</up>")
  catln(3, "<down>")
  catln(4, numbered.idref("parameter", "", g, ".treeModel.allInternalNodeHeights"))
  catln(3, "</down>")
  catln(2, "</upDownOperator>")
  } 
  
  
      
make.upDownOperator.clocks.pops.heights <- function(nofgenes) {
  catln(2, "<upDownOperator", namevalue("scaleFactor", 0.75), 
           namevalue("weight", 30), ">")
           
  catln(3, "<up>")
  for (g in 1:nofgenes) {
    if (g>1) {
      catln(4, numbered.idref("parameter", "", g, ".clock.rate")) 
    }
  }
  catln(4, idref("parameter", "apspnetwork.prior.eventRate")) 
  catln(3, "</up>")
  
  catln(3, "<down>")
  catln(4, idref("alloppSpeciesNetwork", "apspnetwork"))
  catln(4, idref("parameter" , "species.popMean"))
  catln(4, idref("parameter" , "apspnetwork.splitPopSize"))
  for (g in 1:nofgenes) {
    catln(4, numbered.idref("parameter", "", g, ".treeModel.allInternalNodeHeights"))
  }
  catln(3, "</down>")
  catln(2, "</upDownOperator>")
  }
  
  



make.scaleOperator.species.popMean <- function() {
  make.general.scaleOperator(0.9, 5, "species.popMean")
  }
  
   
make.scaleOperator.apspnetwork.splitPopSize <- function() {
  make.general.scaleOperator(0.5, 30, "apspnetwork.splitPopSize")
  }
  
   
make.scaleOperator.apspnetwork.prior.eventRate <- function() {
  make.general.scaleOperator(0.5, 30, "apspnetwork.prior.eventRate")
  }
  


    
make.networkNodeReHeightOperator <- function() {
  catln(2, "<networkNodeReHeight",  namevalue("weight", 30), ">")
  catln(3, idref("alloppSpecies", "alloppSpecies"))
  catln(3, idref("alloppSpeciesNetwork", "apspnetwork"))
  catln(2, "</networkNodeReHeight>")   
  }
  
  
make.sequenceReassignmentOperator <- function() {
  catln(2, "<sequenceReassignment",  namevalue("weight", 30), ">")
  catln(3, idref("alloppSpecies", "alloppSpecies"))
  catln(3, idref("alloppSpeciesNetwork", "apspnetwork"))
  catln(2, "</sequenceReassignment>")   
  }
  
    
make.moveLegsOperator <- function() {
  catln(2, "<moveLegs",  namevalue("weight", 30), ">")
  catln(3, idref("alloppSpecies", "alloppSpecies"))
  catln(3, idref("alloppSpeciesNetwork", "apspnetwork"))
  catln(2, "</moveLegs>")   
  }
      

#############################################################
##########               Logging               ##############
#############################################################
 



